﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace HRPortal
{
    
    /// <summary>
    /// Wrapper class to help with calls to the WCF service when called by the object data sources.
    /// </summary>
    public class HRServiceWrapper
    {

        /// <summary>
        /// Get the list of employees.
        /// </summary>
        /// <returns></returns>
        public List<HRServiceProxy.Employee> ListEmployees()
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();
            try
            {
                return proxy.ListEmployees();
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }

        /// <summary>
        /// Get the specified employee record by ID
        /// </summary>
        /// <param name="ID"></param>
        /// <returns></returns>
        public HRServiceProxy.EmployeeUpdateInfo GetEmployee(string ID)
        {
            if (string.IsNullOrEmpty(ID))
                return null;


            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();
            try
            {
                HRServiceProxy.Employee employee = proxy.GetEmployee(ID);

                return new HRServiceProxy.EmployeeUpdateInfo() 
                    { 
                        ID = employee.ID,
                        BonusPercentage = employee.BonusPercentage,
                        ExtensionData = employee.ExtensionData,
                        FirstName = employee.FirstName,
                        HomeAddress = employee.HomeAddress,
                        LastName = employee.LastName,
                        ManagerID = employee.ManagerID,
                        Position = employee.Position,
                        Salary = employee.Salary,
                        SSN = employee.SSN,
                        Username = employee.Username
                    };
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }


        /// <summary>
        /// Update the employee record using the specified information. Only fields flagged for update via the associated boolean 
        /// property will be updated.
        /// </summary>
        /// <param name="employee"></param>
        public void UpdateEmployee(HRServiceProxy.EmployeeUpdateInfo employee)
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();
            try
            {
                proxy.UpdateEmployee(employee);
                return;
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }


        /// <summary>
        /// Get my employee record.
        /// </summary>
        /// <returns></returns>
        public HRServiceProxy.EmployeeUpdateInfo GetMyEmployeeRecord()
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();
            try
            {
                HRServiceProxy.Employee employee = proxy.GetMyEmployeeRecord();

                return new HRServiceProxy.EmployeeUpdateInfo()
                {
                    ID = employee.ID,
                    BonusPercentage = employee.BonusPercentage,
                    ExtensionData = employee.ExtensionData,
                    FirstName = employee.FirstName,
                    HomeAddress = employee.HomeAddress,
                    LastName = employee.LastName,
                    ManagerID = employee.ManagerID,
                    Position = employee.Position,
                    Salary = employee.Salary,
                    SSN = employee.SSN,
                    Username = employee.Username
                };
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }

        /// <summary>
        /// Update my employee record using the specified information. Only fields flagged for update via the associated boolean 
        /// property will be updated.
        /// </summary>
        /// <param name="employee"></param>
        public void UpdateMyEmployeeRecord(HRServiceProxy.EmployeeUpdateInfo employee)
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();
            try
            {
                proxy.UpdateMyEmployeeRecord(employee);
                return;
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }

        /// <summary>
        /// Get my rights for operations on employee records.
        /// </summary>
        /// <returns></returns>
        public HRServiceProxy.Rights GetMyRights()
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();

            try
            {
                return proxy.GetMyRights();
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }


        /// <summary>
        /// Get my rights for operations on my own employee record.
        /// </summary>
        /// <returns></returns>
        public HRServiceProxy.Rights GetMyRightsToSelf()
        {
            HRServiceProxy.HRServiceClient proxy = new HRPortal.HRServiceProxy.HRServiceClient();

            try
            {
                return proxy.GetMyRightsToSelf();
            }
            finally
            {
                try
                {
                    proxy.Close();
                }
                catch { }
            }
        }

    }
}
