﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace HRPortal
{
    public partial class MyInfo : System.Web.UI.Page
    {
        private HRServiceProxy.Rights _MyRightsToSelf = null;

        protected HRServiceProxy.Rights MyRightsToSelf
        {
            get
            {
                // Could store in ViewState or Session for efficiency.
                // Note: Permissions are checked again on the service layer, so viewstate tampering vulnerability is not a concern.
                // return (HRServiceProxy.Rights)ViewState["RightsToSelf"];

                // Store in member variable and fetch on each load for demo so that changes take affect immediately.
                return _MyRightsToSelf;
            }
            set
            {
                // ViewState["RightsToSelf"] = value;

                _MyRightsToSelf = value;
            }
        }

        
        
        protected void Page_Load(object sender, EventArgs e)
        {
            trErrors.Visible = false;

            // Get the user's rights. This is used during edit operations.
            HRServiceWrapper svc = new HRServiceWrapper();
            this.MyRightsToSelf = svc.GetMyRightsToSelf();

            dvEmployee.Fields.OfType<CommandField>().First().ShowEditButton = this.MyRightsToSelf.UpdateEmployee;

        }




        /// <summary>
        /// Catch this event so that we can handle exceptions from the HR Service.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void dvEmployee_ItemUpdated(object sender, DetailsViewUpdatedEventArgs e)
        {
            if (null != e.Exception)
            {
                if (null != e.Exception.InnerException)
                    lblErrors.Text = e.Exception.InnerException.Message;
                else
                    lblErrors.Text = e.Exception.Message;

                trErrors.Visible = true;
                e.ExceptionHandled = true;
                e.KeepInEditMode = true;
            }
        }


        /// <summary>
        /// Intercept the selected event so that we can set the "Update" flags based on the users rights.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void odsHRGetEmployee_Selected(object sender, ObjectDataSourceStatusEventArgs e)
        {
            HRServiceProxy.EmployeeUpdateInfo ret = (HRServiceProxy.EmployeeUpdateInfo)e.ReturnValue;

            if (e.ReturnValue == null)
                return;

            HRServiceProxy.Rights myRights = this.MyRightsToSelf;

            ret.UpdateBonusPercentage = myRights.SetEmployeeSalary;
            ret.UpdateFirstName = myRights.UpdateEmployee;
            ret.UpdateHomeAddress = myRights.SetEmployeeAddress;
            ret.UpdateLastName = myRights.UpdateEmployee;
            ret.UpdateManagerID = myRights.SetEmployeeManager;
            ret.UpdatePosition = myRights.SetEmployeePosition;
            ret.UpdateSalary = myRights.SetEmployeeSalary;
            ret.UpdateSSN = myRights.SetEmployeeSSN;
            ret.UpdateUsername = myRights.UpdateEmployee;
        }

    }
}
