﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Security;

namespace HRPortalMR
{

    /// <summary>
    /// This demo shows how you can use the ASP.NET Membership and Roles provider for AzMan.
    /// </summary>
    public partial class _Default : System.Web.UI.Page
    {

        protected void Page_Load(object sender, EventArgs e)
        {
            IPrincipal user = HttpContext.Current.User;


            lblName.Text = user.Identity.Name;


            // Uses ASP.NET Membership and Roles API...
            // Pulls the role info from AzMan's policy store.
            // Try puting the user in additional roles to see changes here.
            string[] roles = System.Web.Security.Roles.GetRolesForUser(user.Identity.Name);
            lstRoles.DataSource = roles;

            // Uses the WindowsPrincipal to get the users's group memberships.
            var groups = ((WindowsIdentity)user.Identity).Groups;
            lstWindowsRoles.DataSource = (from g in groups
                                          select GetGroupName(g)).ToList();

            // Do "operation" level checks against AzMan.
            // Note that the AzMan provider for ASP.NET doesn't support "Operations".
            IAuthorizationProvider auth = AuthorizationFactory.GetAuthorizationProvider("HRPortalMR");
            List<string> ops = new List<string>();
            ops.Add("GetEmployee: " + auth.Authorize(user, "O:" + "GetEmployee").ToString());
            ops.Add("ListEmployees: " + auth.Authorize(user, "O:" + "ListEmployees").ToString());
            ops.Add("UpdateEmployee: " + auth.Authorize(user, "O:" + "UpdateEmployee").ToString());
            ops.Add("SetEmployeeSSN: " + auth.Authorize(user, "O:" + "SetEmployeeSSN").ToString());
            ops.Add("...");
            lstOperations.DataSource = ops;

            Page.DataBind();
        }


        /// <summary>
        /// Helper method to translate the SID into a readable name.
        /// </summary>
        /// <param name="sid">The windows SID.</param>
        /// <returns>The name of the user or group represented by the SID.</returns>
        private string GetGroupName(IdentityReference sid)
        {
            try
            {
                return ((NTAccount)sid.Translate(typeof(NTAccount))).Value;
            }
            catch
            {
                return sid.Value;
            }
        }
    }
}
