﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;


namespace HRService
{
    /// <summary>
    /// Populate the Context's Rights properties using IPrincipal.IsInRole("").
    /// Does not use AzMan.
    /// </summary>
    /// <remarks>
    /// This implementation uses hard coded groups and hard coded authorization mapping rules.
    /// </remarks>
    class Auth_NoAzMan
        : IAuthProvider
    {

        private const string ROLE_HR = @".\AM_HR";
        private const string ROLE_HELPDESK = @".\AM_Helpdesk";
        private const string ROLE_USERS = @"Everyone";
        private const string ROLE_ADMINISTRATORS = @".\Administrators";
        private const string ROLE_MANAGERS = @".\AM_Managers";
        
        #region IAuthProvider Members

        public void ApplyPermissions(HRServiceBus.Context ctx)
        {

            IIdentity user = ctx.User;
            IPrincipal p = new WindowsPrincipal((WindowsIdentity)user);
                                    
            ctx.Rights = new HRServiceEntities.Rights();

            // Apply rules by doing "IsInRole()" checks.
            // Note that the group names and the rule logic is hard coded. 
            // Changes require code / test / deploy.

            ctx.Rights.AddEmployee = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.Rights.GetEmployee = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.Rights.GetEmployeeAddress = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_MANAGERS) || p.IsInRole(ROLE_HELPDESK));
            ctx.Rights.GetEmployeeBonus = (p.IsInRole(ROLE_HR));
            ctx.Rights.GetEmployeeSalary = (p.IsInRole(ROLE_HR));
            ctx.Rights.GetEmployeeSSN = (p.IsInRole(ROLE_HR));
            ctx.Rights.ListEmployees = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.Rights.ReinstateEmployees = (p.IsInRole(ROLE_HR));
            ctx.Rights.SetEmployeeAddress = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.Rights.SetEmployeeManager = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.Rights.SetEmployeePosition = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.Rights.SetEmployeeSalary = (p.IsInRole(ROLE_HR));
            ctx.Rights.SetEmployeeSSN = (p.IsInRole(ROLE_HR));
            ctx.Rights.TerminateEmployee = (p.IsInRole(ROLE_HR));
            ctx.Rights.UpdateEmployee = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));


            // -------------------------------------------------------------------
            // Get rights for Self
            // -------------------------------------------------------------------

            ctx.RightsToSelf = new HRServiceEntities.Rights();

            ctx.RightsToSelf.AddEmployee = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.RightsToSelf.GetEmployee = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.GetEmployeeAddress = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.GetEmployeeBonus = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.GetEmployeeSalary = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.GetEmployeeSSN = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.ListEmployees = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.ReinstateEmployees = (p.IsInRole(ROLE_HR));
            ctx.RightsToSelf.SetEmployeeAddress = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
            ctx.RightsToSelf.SetEmployeeManager = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.RightsToSelf.SetEmployeePosition = (p.IsInRole(ROLE_HR) || p.IsInRole(ROLE_HELPDESK));
            ctx.RightsToSelf.SetEmployeeSalary = (p.IsInRole(ROLE_HR));
            ctx.RightsToSelf.SetEmployeeSSN = (p.IsInRole(ROLE_HR));
            ctx.RightsToSelf.TerminateEmployee = (p.IsInRole(ROLE_HR));
            ctx.RightsToSelf.UpdateEmployee = (p.IsInRole(ROLE_USERS) || p.IsInRole(ROLE_ADMINISTRATORS));
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            return;
        }

        #endregion
    }
}
