﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HRServiceEntities;

namespace HRServiceBus
{
    
    public class HRBus
    {

        /// <summary>
        /// List all employees.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights.</param>
        /// <returns>A list of all employees, except my employee record.</returns>
        /// <remarks>The method uses a hard-coded "DAL".</remarks>
        public List<Employee> GetEmployees(Context ctx)
        {
            if (!ctx.Rights.ListEmployees)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "list employees"));


            List<Employee> ret = new List<Employee>();

            ret.Add(this.GetEmployee(ctx, "00001"));
            ret.Add(this.GetEmployee(ctx, "00002"));
            ret.Add(this.GetEmployee(ctx, "00003"));

            return ret;
        }

        /// <summary>
        /// Get a single employee record.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <param name="ID">The employee ID (ex: "00001", "00002", "00003")</param>
        /// <returns>The specified employee record</returns>
        /// <remarks>The method uses a hard-coded "DAL".</remarks>
        public Employee GetEmployee(Context ctx, string ID)
        {
            if (!ctx.Rights.ListEmployees)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "get an employee record"));

            Employee ret = null;

            switch (ID)
            {
                #region Employee 00001

                case "00001":
                    ret = new Employee()
                    {
                        ID = ID,
                        FirstName = "The",
                        LastName = "Man",
                        Username = @"OFFICE\tman",
                        Position = "Executives",
                        ManagerID = null
                    };


                    if (ctx.Rights.GetEmployeeSalary)
                    {
                        ret.Salary = 100000;
                    }
                    else
                    {
                        ret.Salary = null;
                    }


                    if (ctx.Rights.GetEmployeeBonus)
                    {
                        ret.BonusPercentage = 10;
                    }
                    else
                    {
                        ret.BonusPercentage = null;
                    }


                    if (ctx.Rights.GetEmployeeSSN)
                    {
                        ret.SSN = "123-123-1111";
                    }
                    else
                    {
                        ret.Salary = null;
                    }

                    if (ctx.Rights.GetEmployeeAddress)
                    {
                        ret.HomeAddress = new Address()
                        {
                            Street1 = "123 Main St.",
                            City = "Cleveland",
                            State = "OH",
                            Zip = "44017"
                        };
                    }


                    break;

                #endregion

                #region Employee 00002

                case "00002":
                    ret = new Employee()
                    {
                        ID = ID,
                        FirstName = "John",
                        LastName = "Doe",
                        Username = @"OFFICE\jdoe",
                        Position = "Staff",
                        ManagerID = "00001"
                    };


                    if (ctx.Rights.GetEmployeeSalary)
                    {
                        ret.Salary = 50000;
                    }
                    else
                    {
                        ret.Salary = null;
                    }


                    if (ctx.Rights.GetEmployeeBonus)
                    {
                        ret.BonusPercentage = 8;
                    }
                    else
                    {
                        ret.BonusPercentage = null;
                    }


                    if (ctx.Rights.GetEmployeeSSN)
                    {
                        ret.SSN = "123-123-2222";
                    }
                    else
                    {
                        ret.Salary = null;
                    }

                    if (ctx.Rights.GetEmployeeAddress)
                    {
                        ret.HomeAddress = new Address()
                        {
                            Street1 = "1000 College Ave.",
                            City = "Ft. Collins",
                            State = "CO",
                            Zip = "80521"
                        };
                    }


                    break;

                #endregion

                #region Employee 00003

                case "00003":
                    ret = new Employee()
                    {
                        ID = ID,
                        FirstName = "Jane",
                        LastName = "Smith",
                        Username = @"OFFICE\jsmith",
                        Position = "Staff",
                        ManagerID = "00001"
                    };


                    if (ctx.Rights.GetEmployeeSalary)
                    {
                        ret.Salary = 50000;
                    }
                    else
                    {
                        ret.Salary = null;
                    }


                    if (ctx.Rights.GetEmployeeBonus)
                    {
                        ret.BonusPercentage = 8;
                    }
                    else
                    {
                        ret.BonusPercentage = null;
                    }


                    if (ctx.Rights.GetEmployeeSSN)
                    {
                        ret.SSN = "123-123-3333";
                    }
                    else
                    {
                        ret.Salary = null;
                    }

                    if (ctx.Rights.GetEmployeeAddress)
                    {
                        ret.HomeAddress = new Address()
                        {
                            Street1 = "15 Washington St.",
                            City = "Cleveland",
                            State = "OH",
                            Zip = "44017"
                        };
                    }


                    break;

                #endregion

                default:
                    throw new ArgumentOutOfRangeException();
            }
           
            return ret;
        }


        /// <summary>
        /// Get the current user's employee record.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <param name="ID"></param>
        /// <returns></returns>
        /// <remarks>The method uses a hard-coded "DAL".</remarks>
        public Employee GetMyEmployeeRecord(Context ctx)
        {
            if (!ctx.RightsToSelf.GetEmployee)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "get an employee record"));

            Employee ret = null;

            #region Employee 10000

            ret = new Employee()
            {
                ID = "10000",
                FirstName = "My",
                LastName = "Self",
                Username = ctx.User.Name,
                Position = "Staff",
                ManagerID = "00001"
            };


            if (ctx.RightsToSelf.GetEmployeeSalary)
            {
                ret.Salary = 50000;
            }
            else
            {
                ret.Salary = null;
            }


            if (ctx.RightsToSelf.GetEmployeeBonus)
            {
                ret.BonusPercentage = 5;
            }
            else
            {
                ret.BonusPercentage = null;
            }


            if (ctx.RightsToSelf.GetEmployeeSSN)
            {
                ret.SSN = "999-999-9999";
            }
            else
            {
                ret.Salary = null;
            }

            if (ctx.RightsToSelf.GetEmployeeAddress)
            {
                ret.HomeAddress = new Address()
                {
                    Street1 = "123 Self St.",
                    City = "Medina",
                    State = "OH",
                    Zip = "44111"
                };
            }

            #endregion

            return ret;
        }


        /// <summary>
        /// Update the specified employee record. 
        /// Only fields with the UpdateXXX field set to True are actually updated.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <exception cref="System.ApplicationException">
        /// Throws an application exception if the user doesn't have permission.
        /// </exception>
        /// <remarks>
        /// For demo purposes, this method checks permissions, but does not 
        /// actually change the employee record.
        /// </remarks>
        public void UpdateEmployee(Context ctx, EmployeeUpdateInfo info)
        {
            if (!ctx.Rights.UpdateEmployee)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "update employee records"));

            if (info.UpdateManagerID && !ctx.Rights.SetEmployeeManager)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's manager"));

            if (info.UpdatePosition && !ctx.Rights.SetEmployeePosition)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's position"));

            if (info.UpdateSalary && !ctx.Rights.SetEmployeeSalary)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's salary"));

            if (info.UpdateSSN && !ctx.Rights.SetEmployeeSSN)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's SSN"));


            // NOT IMPLEMENTED
            // Update the actual data store.


            return;
        }


        /// <summary>
        /// Update the specified employee record. 
        /// Only fields with the UpdateXXX field set to True are actually updated.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <exception cref="System.ApplicationException">
        /// Throws an application exception if the user doesn't have permission.
        /// </exception>
        /// <remarks>For demo purposes, this method checks permissions, but does not actually change the employee record.</remarks>
        public void UpdateMyEmployeeRecord(Context ctx, EmployeeUpdateInfo info)
        {
            if (string.Compare(info.Username, ctx.User.Name, true) != 0)
                throw new ApplicationException("The specified employee record is not your record.");
            
            if (!ctx.RightsToSelf.UpdateEmployee)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "update employee records"));

            if (info.UpdateManagerID && !ctx.RightsToSelf.SetEmployeeManager)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's manager"));

            if (info.UpdatePosition && !ctx.RightsToSelf.SetEmployeePosition)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's position"));

            if (info.UpdateSalary && !ctx.RightsToSelf.SetEmployeeSalary)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's salary"));

            if (info.UpdateSSN && !ctx.RightsToSelf.SetEmployeeSSN)
                throw new ApplicationException(string.Format("{0} is not authorized to {1}.", ctx.User.Name, "set employee's SSN"));

            
            // NOT IMPLEMENTED
            // Update the actual data store.


            return;
        }

        /// <summary>
        /// Get the rights for the specified user.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <returns>An instance of the rights class, containing the permissions of the user.</returns>
        public Rights GetRights(Context ctx)
        {
            return ctx.Rights;
        }


        /// <summary>
        /// Get the rights to "self" operations.
        /// </summary>
        /// <param name="ctx">The context object, containing the user identity and rights</param>
        /// <returns>An instance of the rights class, containing the permissions of the user specific to operations on their own employee record.</returns>
        public Rights GetRightsToSelf(Context ctx)
        {
            return ctx.RightsToSelf;
        }

    }
}
