﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Security.Principal;

namespace HRServiceTests
{
    /// <summary>
    /// Test the business layer's methods.
    /// </summary>
    [TestClass]
    public class BizTests
    {

        #region Members and Constructors

        public BizTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #endregion


        /// <summary>
        /// Check that we can list employees if we have the ListEmployees permission.
        /// </summary>
        [TestMethod]
        public void ListEmployees()
        {
            // Create the context with the rights that we need for this test.
            HRServiceBus.Context ctx = new HRServiceBus.Context(WindowsIdentity.GetCurrent());

            ctx.Rights = new HRServiceEntities.Rights()
            {
                ListEmployees = true
            };

            // List the employees.
            HRServiceBus.HRBus bus = new HRServiceBus.HRBus();

            List<HRServiceEntities.Employee> employees = bus.GetEmployees(ctx);

            Assert.IsNotNull(employees, "No employees returned.");
            Assert.IsTrue(employees.Count > 0, "No employees returned.");
        }


        /// <summary>
        /// Check that the service returns an exception if we don't have UpdateEmployee permission.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(System.ApplicationException))]
        public void UpdateEmployeeWithoutPermissionFails()
        {
            // Create the context with the rights that we need for this test.
            HRServiceBus.Context ctx = new HRServiceBus.Context(WindowsIdentity.GetCurrent());

            ctx.Rights = new HRServiceEntities.Rights()
            {
                UpdateEmployee = false
            };

            // Change the employee
            HRServiceEntities.EmployeeUpdateInfo updatedEmployee = new HRServiceEntities.EmployeeUpdateInfo();
            updatedEmployee.ID = "00001";
            updatedEmployee.FirstName = "John";
            updatedEmployee.UpdateFirstName = true;
            
            // Save the changes.
            HRServiceBus.HRBus bus = new HRServiceBus.HRBus();
            bus.UpdateEmployee(ctx, updatedEmployee);

            Assert.Fail("Should have gotten an exception.");
        }


        /// <summary>
        /// Check that the service returns an exception if we don't have UpdateEmployeeSSN permission.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(System.ApplicationException))]
        public void UpdateEmployeeSSNWithoutPermissionFails()
        {
            // Create the context with the rights that we need for this test.
            HRServiceBus.Context ctx = new HRServiceBus.Context(WindowsIdentity.GetCurrent());

            ctx.Rights = new HRServiceEntities.Rights()
            {
                UpdateEmployee = true,
                SetEmployeeSSN = false
            };

            // Change the employee
            HRServiceEntities.EmployeeUpdateInfo updatedEmployee = new HRServiceEntities.EmployeeUpdateInfo();
            updatedEmployee.ID = "00001";
            updatedEmployee.SSN = "111-11-1111";
            updatedEmployee.UpdateSSN = true;

            // Save the changes.
            HRServiceBus.HRBus bus = new HRServiceBus.HRBus();
            bus.UpdateEmployee(ctx, updatedEmployee);

            Assert.Fail("Should have gotten an exception.");
        }
    }
}
